#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 20212-07-08
# File  : inv_opengear_serial_line
#
# inventory of opengear console servers serial lines
#
# 2023-02-20: moved gui files from ~/local/share/check_mk/... to ~/local/lib/check_mk...
# 2023-06-16: removed unused inventory_ruleset_name and params
# 2024-05-10: fixed wrong inventory function name
#             integrated .1.3.6.1.4.1.25049.17.2.1 and .1.3.6.1.4.1.25049.10.19.2.2.1

from collections.abc import Mapping, Sequence
from typing import List

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    OIDEnd,
    SNMPTree,
    TableRow,
    register,
    startswith,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    InventoryResult,
    StringTable,
)

_opengear_state = {
    '0': 'off',
    '1': 'on',
    '2': 'off',
}

_opengear_log_level = {
    '1': 'disabled',
    '2': 'connect',
    '3': 'inputAndOutput',
    '4': 'inputOnly',
    '5': 'outputOnly',
}

_opengear_mode = {
    '1': 'none',
    '2': 'console',
    '3': 'sdt',
    '4': 'terminal',
    '5': 'bridge',
}

_opengear_flow_control = {
    '1': 'none',
    '2': 'hardware',
    '3': 'software',
}

_opengear_stop_bits = {
    '1': 'one',
    '2': 'two',
    '3': 'oneAndAHalf',
}

_opengear_parity = {
    '1': 'none',
    '2': 'odd',
    '3': 'even',
    '4': 'mark',
    '5': 'space',
}

SECTION = Sequence[Mapping[str, Mapping[str, str]]]


def parse_inv_opengear_serial_lines(string_table: List[StringTable]) -> SECTION | None:
    try:
        raw_data, raw_data_v2 = string_table
    except ValueError:
        return

    if not raw_data:
        raw_data = raw_data_v2

    section = []

    for line in raw_data:
        try:
            index, label, speed, databits, parity, stopbits, flowcontrol, mode, loglevel, dcd, dtr, dsr, cts, rts = line
        except ValueError:
            return

        entry = {
            'key_columns': {
                'index': f'{int(index):02}'
            },
            'inventory_columns': {
                'label': label,
                'speed': speed,
                'databits': databits,
                'parity': _opengear_parity.get(parity, parity),
                'stopbits': _opengear_stop_bits.get(stopbits, stopbits),
                'flowcontrol': _opengear_flow_control.get(flowcontrol, flowcontrol),
                'mode': _opengear_mode.get(mode, mode),
                'loglevel': _opengear_log_level.get(loglevel, loglevel),
                'cts': _opengear_state[cts],

                'dcd': _opengear_state[dcd],
                'dtr': _opengear_state[dtr],
                'rts': _opengear_state[rts],
            },
            'status_columns': {
            }
        }

        section.append(entry)

    return section


def inv_opengear_serial_lines(section: SECTION) -> InventoryResult:
    path = ['hardware', 'serial_lines']

    for entry in section:
        yield TableRow(
            path=path,
            key_columns=entry['key_columns'],
            inventory_columns=entry['inventory_columns'],
            status_columns=entry['status_columns']
        )


register.snmp_section(
    name='inv_opengear_serial_lines',
    parse_function=parse_inv_opengear_serial_lines,
    fetch=[
        SNMPTree(
            base='.1.3.6.1.4.1.25049.17.2.1',  # OG-STATUSv2-MIB:orgSerialPortEntry
            oids=[
                OIDEnd(),  # line index
                '2',  # ogSerialPortLabel
                '3',  # ogSerialPortSpeed
                '4',  # ogSerialPortDataBits
                '5',  # ogSerialPortParity
                '6',  # ogSerialPortStopBits
                '7',  # ogSerialPortFlowControl
                '8',  # ogSerialPortMode
                '9',  # ogSerialPortLogLevel
                # '10',  # ogSerialPortRxBytes
                # '11',  # ogSerialPortTxBytes
                # '12',  # ogSerialPortFramingErrors
                # '13',  # ogSerialPortParityErrors
                # '14',  # ogSerialPortOverrunErrors
                # '15',  # ogSerialPortBreaks
                '16',  # ogSerialPortDCD
                '17',  # ogSerialPortDTR
                '18',  # ogSerialPortDSR
                '19',  # ogSerialPortCTS
                '20',  # ogSerialPortRTS
            ]
        ),
        SNMPTree(
            base='.1.3.6.1.4.1.25049.10.19.2.2.1',  #
            oids=[
                OIDEnd(),  # line index
                '2',  # ogSerialPortLabel
                '3',  # ogSerialPortSpeed
                '4',  # ogSerialPortDataBits
                '5',  # ogSerialPortParity
                '6',  # ogSerialPortStopBits
                '7',  # ogSerialPortFlowControl
                '8',  # ogSerialPortMode
                '9',  # ogSerialPortLogLevel
                # '10',  # ogSerialPortRxBytes
                # '11',  # ogSerialPortTxBytes
                # '12',  # ogSerialPortFramingErrors
                # '13',  # ogSerialPortParityErrors
                # '14',  # ogSerialPortOverrunErrors
                # '15',  # ogSerialPortBreaks
                '16',  # ogSerialPortDCD
                '17',  # ogSerialPortDTR
                '18',  # ogSerialPortDSR
                '19',  # ogSerialPortCTS
                '20',  # ogSerialPortRTS
            ]
        ),
    ],

    detect=startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.25049'),
)

register.inventory_plugin(
    name='inv_opengear_serial_lines',
    inventory_function=inv_opengear_serial_lines,
)
